﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml;

namespace NicoCheck
{
    class Program
    {
        // メイン処理
        static void Main(string[] args)
        {
            MainWork();
        }

        // 個々のマイリスト情報をあらわすクラス
        private class NicoInfo
        {
            public int ID;
            public DateTime Time;
            public string Title;

            // コンストラクタ
            public NicoInfo(int id, DateTime time, string title)
            {
                this.ID = id;
                this.Time = time;
                this.Title = title;
            }

            // NicoInfo同士を更新時刻で比較するメソッド
            public static int CompareByTime(NicoInfo x, NicoInfo y)
            {
                // 更新時刻の新しいものを優先する
                int result = (y.Time).CompareTo(x.Time);
                if (result == 0)
                {
                    // 時刻が同じなら番号の小さいものを優先する
                    result = (x.ID).CompareTo(y.ID);
                }
                return result;
            }
        }

        // すべてのマイリスト情報を保持する可変長配列
        private static List<NicoInfo> Data;

        // データ
        private static string[] Lines =
        {
            "14063456 ノベマス短編（ハリアーP）",
            "15008034 アイドルinエルシア（がんびっとP）",
            "15973681 小鳥さんのGM奮闘記（ブリッツＰ）",
            "17707796 続・小鳥さんのＧＭ奮闘記（ブリッツＰ）",
            "20845122 さらばダーレスブルグ（コオロギ）",
            "21484018 東方紅華仙（Hex-a-bloom）",
            "24774299 続々・小鳥さんのＧＭ奮闘記（ブリッツＰ）",
            "26519132 バルナッド・キャンペーン（DIOP）",
            "30443658 東方～夢遊病～（∫0∫（東方手書き作者））",
            "31053477 紅美鈴と不敵な仲間たち（緑青）",
            "31231052 自作シナリオ『沼男は誰だ？』（ma34）",
            "31587847 本当にあったSAN値が下がるクトゥルフTRPG（二人組を作る）",
            "32746620 小鳥さんのGM奮闘記R（ブリッツＰ）",
            "33435425 風変りな旅行（二人組を作る）",
            "34130035 絵理のDnD5th／絵理ゴハン（がんびっとR）",
            "34167065 闇をゆく者達の宴（久海　夏輝）",
            "35021252 旅籠の日雇い冒険者達（シュレ迷子Ｐ）",
            "35457309 さとりとEX三人娘のアリアンロッド2E（諏訪子に祟られたい人）",
            "36539028 雪歩と愉快な仲間たちのRPてんこもり特殊蛮族卓（完熟Ｐ）",
            "36817873 ラフヘローの結末（二人組を作る）",
            "39766537 レッツゴー!!関西探偵団（二人組を作る）",
            "40534988 零時に魔法が解ける頃（びんりつＰ）",
            "42946540 でんの子Pの曲（でんの子P）",
            "45400631 脳髄倶楽部（二人組を作る）",
            "45981991 地味ぃに進む旧ソードワールド（さっちゃん）",
            "48197245 765プロのちょっと不思議なSW2.0（モコキッカー）",
            "49317018 沼男は誰だ？（久海　夏輝）",
            "50819396 無所属冒険者の大陸漫遊卓（焼肉）",
            "53639528 蠢く島（もりゆき）",
            "53670830 ゆかゆかのりこが霧の街に挑むようです（まっきー）",
            "55008544 父を探して（二人組を作る）",
            "55270800 魔法なんて無かった（混沌Ｐ）",
            "56111374 剣の国の魔法戦士チルノ（さっちゃん）",
            "56160648 ずんだ餅が世界を救うSW2.0（ライアン）",
            "57250859 嘘みたいに上手くいくクトゥルフ神話TRPG（うそうま）",
            "57441802 よいこの琴葉クトゥルフ（となり）",
            "57578029 邪眼の姫の物語（混沌Ｐ）",
            "58098001 続々々・小鳥さんのＧＭ奮闘記（ブリッツＰ）",
            "58562771 第19回MMD杯本戦1（19杯運営）",
            "59108377 茜ちゃん迷宮に挑む（noyuki）",
            "59334028 こずきりのソード・ワールド2.0（混沌Ｐ）",
            "59379095 ボイスロイドたちが読む禁断の書物（Cronish）",
            "59406558 セイギノミカタ（もりゆき）",
            "59805888 自作動画（yuima）",
        };

        // マイリストチェック処理の本体
        private static void MainWork()
        {
            // データを読み込む
            Data = new List<NicoInfo>();
            foreach (string line in Lines)
            {
                Match m = Regex.Match(line, @"^(\d+)\s+(.*)");
                if (m.Success)
                {
                    Data.Add(new NicoInfo(
                        int.Parse(m.Groups[1].Value),
                        DateTime.MinValue,
                        m.Groups[2].Value
                    ));
                }
            }

            // マイリスト更新チェック開始
            Console.WriteLine("チェック開始");
            foreach (NicoInfo info in Data)
            {
                try
                {
                    // ニコニコ動画からマイリスト情報をダウンロードして検査
                    Console.WriteLine("チェック中: [" + info.ID + "] " + info.Title);
                    info.Time = GetUpdateTime(info.ID);

                    // サーバーに負荷がかかると悪いのでちょっと待つ
                    Thread.Sleep(250);
                }
                catch (Exception ex)
                {
                    // マイリストが非公開などの場合、エラー発生
                    Console.WriteLine("エラー: [" + info.ID + "]: " + ex.Message);
                }
            }
            Console.WriteLine("チェック完了\n");

            // 更新時刻が新しい順にマイリストをソートする
            Data.Sort(NicoInfo.CompareByTime);

            // HTMLファイルを作ってアップロードする
            try
            {
                MakeHtml("nico.html");
                UploadFile("nico.html");
            }
            catch (Exception ex)
            {
                Console.WriteLine("エラー: " + ex.Message);
            }
        }

        // ニコ動マイリストに含まれる最新動画の投稿時刻を返す
        // 引数：ニコ動画マイリスト番号
        private static DateTime GetUpdateTime(int id)
        {
            // マイリスト番号から、対応するRSSのURLを得る
            string url = "http://www.nicovideo.jp/mylist/" + id + "?rss=2.0"
                + "&nomemo=1"
                + "&nothumbnail=1"
                + "&nodescription=1";

            // RSSをローカルPCにダウンロードする
            string localFolder = "./mylist";
            string localFile = localFolder + "/" + id + ".xhtml";
            Directory.CreateDirectory(localFolder);
            WebClient client = new WebClient();
            client.DownloadFile(url, localFile);

            // ダウンロードしたファイルからRSSを読み出す
            string xmlText = File.ReadAllText(localFile);

            // XMLとして解析する
            XmlDocument xml = new XmlDocument();
            xml.LoadXml(xmlText);
            XmlElement rss = xml.DocumentElement;

            // 全動画情報をスキャンし、最新の投稿時刻を得る
            DateTime lastdate = DateTime.MinValue;
            XmlElement channel = rss["channel"];

            foreach (XmlElement child in channel.ChildNodes)
            {
                if (child.Name == "item")
                {
                    DateTime date = ParseDate(child["description"].InnerText);
                    if (date > lastdate)
                    {
                        lastdate = date;
                    }
                }
            }

            return lastdate;
        }

        // 動画ごとのdescription要素から投降日時を得る
        // 引数：ニコ動マイリストのRSSに含まれる動画ごとのdescription要素
        // 戻値：動画の投稿時刻（DateTime型）
        private static DateTime ParseDate(string description)
        {
            Match m = Regex.Match(description,
                @"""nico-info-date"">(\d\d\d\d).(\d\d).(\d\d). (\d\d).(\d\d).(\d\d)");

            int year = int.Parse(m.Groups[1].Value);
            int mon = int.Parse(m.Groups[2].Value);
            int day = int.Parse(m.Groups[3].Value);
            int hour = int.Parse(m.Groups[4].Value);
            int min = int.Parse(m.Groups[5].Value);
            int sec = int.Parse(m.Groups[6].Value);

            return new DateTime(year, mon, day, hour, min, sec);
        }

        // アップロード用のHTMLファイルを作る
        private static void MakeHtml(string fileName)
        {
            Console.WriteLine("HTMLファイル作成中: " + fileName);

            StreamWriter file = new StreamWriter(fileName, false, Encoding.UTF8);
            file.Write(""
                + "<!DOCTYPE html>\n"
                + "<html lang=\"ja\">\n"
                + "<head>\n"
                + "<meta charset=\"utf-8\">\n"
                + "<meta name=\"viewport\" content=\"width=device-width\">\n"
                + "<title>視聴中ニコニコ動画</title>\n"
                + "<link rel=\"stylesheet\" type=\"text/css\" href=\"style.css\">\n"
                + "</head>\n"
                + "<body>\n"
                + "<div id=\"main\">\n"
                + "<ul>\n"
            );
            foreach (NicoInfo info in Data)
            {
                file.Write(
                    "<li><a href=\"http://www.nicovideo.jp/mylist/{0}#+sort=6\">[{1}] {2}</a></li>\n",
                    info.ID, info.Time.ToString("yyyy/MM/dd"), info.Title);
            }
            file.Write(""
                + "</ul>\n"
                + "</div>\n"
                + "</body>\n"
                + "</html>\n"
            );
            file.Close();

            Console.WriteLine("HTMLファイル作成完了");
        }

        // ファイルをFTPでアップロードする
        private static void UploadFile(string fileName)
        {
            Console.WriteLine("FTPでアップロード中: " + fileName);

            string host = "s269.xrea.com"; // 自分のFTPサーバ
            string user = "***UserName***"; // 自分のFTPユーザ名に変えてくだされ
            string pass = "***Password***"; // 自分のFTPパスワードに変えてちょ
            string upfolder = "/public_html/yuima"; // サーバのアップロード場所

            WebClient client = new WebClient();
            ; // HACK to avoid IDE0017
            client.Credentials = new NetworkCredential(user, pass);
            string url = string.Format("ftp://{0}{1}/{2}", host, upfolder, fileName);
            client.UploadFile(url, fileName);

            Console.WriteLine("FTPでアップロード完了");
        }

    }
}
